@@ -14,10 +14,7 @@
 %                                                                             %
 %                             Software Design                                 %
 %                                  Cristy                                     %
-%                              January 2003                                   %
-%                                                                             %
-%                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,6 +42,7 @@
 #include "MagickCore/exception.h"
 #include "MagickCore/exception-private.h"
 #include "MagickCore/geometry.h"
+#include "MagickCore/geometry-private.h"
 #include "MagickCore/image-private.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/pixel-accessor.h"
@@ -52,6 +50,113 @@
 #include "MagickCore/string-private.h"
 #include "MagickCore/token.h"
 
+/*
+  Define declarations.
+*/
+#define MagickPagesize(name,geometry) { (name), sizeof(name)-1, (geometry) }
+
+/*
+  Structure declarations.
+*/
+typedef struct _PageInfo
+{
+  const char
+    name[12];
+
+  size_t
+    extent;
+
+  const char
+    geometry[10];
+} PageInfo;
+
+static const PageInfo
+  Pagesizes[] =
+  {
+    MagickPagesize("4x6", "288x432"),
+    MagickPagesize("5x7", "360x504"),
+    MagickPagesize("7x9", "504x648"),
+    MagickPagesize("8x10", "576x720"),
+    MagickPagesize("9x11", "648x792"),
+    MagickPagesize("9x12", "648x864"),
+    MagickPagesize("10x13", "720x936"),
+    MagickPagesize("10x14", "720x1008"),
+    MagickPagesize("11x17", "792x1224"),
+    MagickPagesize("4A0", "4768x6741"),
+    MagickPagesize("2A0", "3370x4768"),
+    MagickPagesize("a0", "2384x3370"),
+    MagickPagesize("a1", "1684x2384"),
+    MagickPagesize("a2", "1191x1684"),
+    MagickPagesize("a3", "842x1191"),
+    MagickPagesize("a4", "595x842"),
+    MagickPagesize("a4small", "595x842"),
+    MagickPagesize("a5", "420x595"),
+    MagickPagesize("a6", "298x420"),
+    MagickPagesize("a7", "210x298"),
+    MagickPagesize("a8", "147x210"),
+    MagickPagesize("a9", "105x147"),
+    MagickPagesize("a10", "74x105"),
+    MagickPagesize("archa", "648x864"),
+    MagickPagesize("archb", "864x1296"),
+    MagickPagesize("archC", "1296x1728"),
+    MagickPagesize("archd", "1728x2592"),
+    MagickPagesize("arche", "2592x3456"),
+    MagickPagesize("b0", "2920x4127"),
+    MagickPagesize("b1", "2064x2920"),
+    MagickPagesize("b10", "91x127"),
+    MagickPagesize("b2", "1460x2064"),
+    MagickPagesize("b3", "1032x1460"),
+    MagickPagesize("b4", "729x1032"),
+    MagickPagesize("b5", "516x729"),
+    MagickPagesize("b6", "363x516"),
+    MagickPagesize("b7", "258x363"),
+    MagickPagesize("b8", "181x258"),
+    MagickPagesize("b9", "127x181"),
+    MagickPagesize("c0", "2599x3676"),
+    MagickPagesize("c1", "1837x2599"),
+    MagickPagesize("c2", "1298x1837"),
+    MagickPagesize("c3", "918x1296"),
+    MagickPagesize("c4", "649x918"),
+    MagickPagesize("c5", "459x649"),
+    MagickPagesize("c6", "323x459"),
+    MagickPagesize("c7", "230x323"),
+    MagickPagesize("csheet", "1224x1584"),
+    MagickPagesize("dsheet", "1584x2448"),
+    MagickPagesize("esheet", "2448x3168"),
+    MagickPagesize("executive", "540x720"),
+    MagickPagesize("flsa", "612x936"),
+    MagickPagesize("flse", "612x936"),
+    MagickPagesize("folio", "612x936"),
+    MagickPagesize("halfletter", "396x612"),
+    MagickPagesize("isob0", "2835x4008"),
+    MagickPagesize("isob1", "2004x2835"),
+    MagickPagesize("isob10", "88x125"),
+    MagickPagesize("isob2", "1417x2004"),
+    MagickPagesize("isob3", "1001x1417"),
+    MagickPagesize("isob4", "709x1001"),
+    MagickPagesize("isob5", "499x709"),
+    MagickPagesize("isob6", "354x499"),
+    MagickPagesize("isob7", "249x354"),
+    MagickPagesize("isob8", "176x249"),
+    MagickPagesize("isob9", "125x176"),
+    MagickPagesize("jisb0", "1030x1456"),
+    MagickPagesize("jisb1", "728x1030"),
+    MagickPagesize("jisb2", "515x728"),
+    MagickPagesize("jisb3", "364x515"),
+    MagickPagesize("jisb4", "257x364"),
+    MagickPagesize("jisb5", "182x257"),
+    MagickPagesize("jisb6", "128x182"),
+    MagickPagesize("ledger", "1224x792"),
+    MagickPagesize("legal", "612x1008"),
+    MagickPagesize("letter", "612x792"),
+    MagickPagesize("lettersmall", "612x792"),
+    MagickPagesize("monarch", "279x540"),
+    MagickPagesize("quarto", "610x780"),
+    MagickPagesize("statement", "396x612"),
+    MagickPagesize("tabloid", "792x1224"),
+    MagickPagesize("", "")
+  };
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -71,11 +176,11 @@
 %
 %  The value must form a proper geometry style specification of WxH+X+Y
 %  of integers only, and values can not be separated by comma, colon, or
-%  slash charcaters.  See ParseGeometry() below.
+%  slash characters.  See ParseGeometry() below.
 %
 %  Offsets may be prefixed by multiple signs to make offset string
 %  substitutions easier to handle from shell scripts.
-%  For example: "-10-10", "-+10-+10", or "+-10+-10" will generate negtive
+%  For example: "-10-10", "-+10-+10", or "+-10+-10" will generate negative
 %  offsets, while "+10+10", "++10++10", or "--10--10" will generate positive
 %  offsets.
 %
@@ -351,7 +456,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetPageGeometry() replaces any page mneumonic with the equivalent size in
+%  GetPageGeometry() replaces any page mnemonic with the equivalent size in
 %  picas.
 %
 %  The format of the GetPageGeometry method is:
@@ -367,106 +472,6 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
 */
 MagickExport char *GetPageGeometry(const char *page_geometry)
 {
-#define MagickPageSize(name,geometry) { (name), sizeof(name)-1, (geometry) }
-
-  typedef struct _PageInfo
-  {
-    const char
-      name[12];
-
-    size_t
-      extent;
-
-    const char
-      geometry[10];
-  } PageInfo;
-
-  static const PageInfo
-    PageSizes[] =
-    {
-      MagickPageSize("4x6", "288x432"),
-      MagickPageSize("5x7", "360x504"),
-      MagickPageSize("7x9", "504x648"),
-      MagickPageSize("8x10", "576x720"),
-      MagickPageSize("9x11", "648x792"),
-      MagickPageSize("9x12", "648x864"),
-      MagickPageSize("10x13", "720x936"),
-      MagickPageSize("10x14", "720x1008"),
-      MagickPageSize("11x17", "792x1224"),
-      MagickPageSize("4A0", "4768x6741"),
-      MagickPageSize("2A0", "3370x4768"),
-      MagickPageSize("a0", "2384x3370"),
-      MagickPageSize("a1", "1684x2384"),
-      MagickPageSize("a2", "1191x1684"),
-      MagickPageSize("a3", "842x1191"),
-      MagickPageSize("a4", "595x842"),
-      MagickPageSize("a4small", "595x842"),
-      MagickPageSize("a5", "420x595"),
-      MagickPageSize("a6", "298x420"),
-      MagickPageSize("a7", "210x298"),
-      MagickPageSize("a8", "147x210"),
-      MagickPageSize("a9", "105x147"),
-      MagickPageSize("a10", "74x105"),
-      MagickPageSize("archa", "648x864"),
-      MagickPageSize("archb", "864x1296"),
-      MagickPageSize("archC", "1296x1728"),
-      MagickPageSize("archd", "1728x2592"),
-      MagickPageSize("arche", "2592x3456"),
-      MagickPageSize("b0", "2920x4127"),
-      MagickPageSize("b1", "2064x2920"),
-      MagickPageSize("b10", "91x127"),
-      MagickPageSize("b2", "1460x2064"),
-      MagickPageSize("b3", "1032x1460"),
-      MagickPageSize("b4", "729x1032"),
-      MagickPageSize("b5", "516x729"),
-      MagickPageSize("b6", "363x516"),
-      MagickPageSize("b7", "258x363"),
-      MagickPageSize("b8", "181x258"),
-      MagickPageSize("b9", "127x181"),
-      MagickPageSize("c0", "2599x3676"),
-      MagickPageSize("c1", "1837x2599"),
-      MagickPageSize("c2", "1298x1837"),
-      MagickPageSize("c3", "918x1296"),
-      MagickPageSize("c4", "649x918"),
-      MagickPageSize("c5", "459x649"),
-      MagickPageSize("c6", "323x459"),
-      MagickPageSize("c7", "230x323"),
-      MagickPageSize("csheet", "1224x1584"),
-      MagickPageSize("dsheet", "1584x2448"),
-      MagickPageSize("esheet", "2448x3168"),
-      MagickPageSize("executive", "540x720"),
-      MagickPageSize("flsa", "612x936"),
-      MagickPageSize("flse", "612x936"),
-      MagickPageSize("folio", "612x936"),
-      MagickPageSize("halfletter", "396x612"),
-      MagickPageSize("isob0", "2835x4008"),
-      MagickPageSize("isob1", "2004x2835"),
-      MagickPageSize("isob10", "88x125"),
-      MagickPageSize("isob2", "1417x2004"),
-      MagickPageSize("isob3", "1001x1417"),
-      MagickPageSize("isob4", "709x1001"),
-      MagickPageSize("isob5", "499x709"),
-      MagickPageSize("isob6", "354x499"),
-      MagickPageSize("isob7", "249x354"),
-      MagickPageSize("isob8", "176x249"),
-      MagickPageSize("isob9", "125x176"),
-      MagickPageSize("jisb0", "1030x1456"),
-      MagickPageSize("jisb1", "728x1030"),
-      MagickPageSize("jisb2", "515x728"),
-      MagickPageSize("jisb3", "364x515"),
-      MagickPageSize("jisb4", "257x364"),
-      MagickPageSize("jisb5", "182x257"),
-      MagickPageSize("jisb6", "128x182"),
-      MagickPageSize("ledger", "1224x792"),
-      MagickPageSize("legal", "612x1008"),
-      MagickPageSize("letter", "612x792"),
-      MagickPageSize("lettersmall", "612x792"),
-      MagickPageSize("monarch", "279x540"),
-      MagickPageSize("quarto", "610x780"),
-      MagickPageSize("statement", "396x612"),
-      MagickPageSize("tabloid", "792x1224")
-    };
-
   char
     page[MagickPathExtent];
 
@@ -474,14 +479,15 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
     i;
 
   assert(page_geometry != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
   (void) CopyMagickString(page,page_geometry,MagickPathExtent);
-  for (i=0; i < (ssize_t) (sizeof(PageSizes)/sizeof(PageSizes[0])); i++)
+  for (i=0; *Pagesizes[i].name != '\0'; i++)
   {
     int
       status;
 
-    status=LocaleNCompare(PageSizes[i].name,page_geometry,PageSizes[i].extent);
+    status=LocaleNCompare(Pagesizes[i].name,page_geometry,Pagesizes[i].extent);
     if (status == 0)
       {
         MagickStatusType
@@ -491,10 +497,10 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
           geometry;
 
         /*
-          Replace mneumonic with the equivalent size in dots-per-inch.
+          Replace mnemonic with the equivalent size in dots-per-inch.
         */
         (void) FormatLocaleString(page,MagickPathExtent,"%s%.80s",
-          PageSizes[i].geometry,page_geometry+PageSizes[i].extent);
+          Pagesizes[i].geometry,page_geometry+Pagesizes[i].extent);
         flags=GetGeometry(page,&geometry.x,&geometry.y,&geometry.width,
           &geometry.height);
         if ((flags & GreaterValue) == 0)
@@ -689,6 +695,52 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
 %                                                                             %
 %                                                                             %
 %                                                                             %
++  L i s t P a g e s i z e s                                                  %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ListPagesizes() lists the pagesizes and their associated geometry.
+%
+%  The format of the ListPagesizes method is:
+%
+%      MagickBooleanType ListPagesizes(FILE *file,ExceptionInfo *exception)
+%
+%  A description of each parameter follows.
+%
+%    o file:  An pointer to the output FILE.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickBooleanType ListPagesizes(FILE *file,
+  ExceptionInfo *magick_unused(exception))
+{
+#define MaxMagickSpaces  ((int) sizeof(Pagesizes[0].name))
+
+  const char
+    *spacer = "                    ";
+
+  ssize_t
+    i;
+
+  magick_unreferenced(exception);
+  if (file == (FILE *) NULL)
+    file=stdout;
+  (void) FormatLocaleFile(file,"\nPagesize    Geometry \n");
+  (void) FormatLocaleFile(file,"---------------------\n");
+  for (i=0; *Pagesizes[i].name != '\0'; i++)
+    (void) FormatLocaleFile(file,"%s%.*s%s\n",Pagesizes[i].name,
+      MaxMagickSpaces-(int) Pagesizes[i].extent,spacer,Pagesizes[i].geometry);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P a r s e A b s o l u t e G e o m e t r y                                 %
 %                                                                             %
 %                                                                             %
@@ -845,11 +897,11 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
 %  relative to the values.
 %
 %  Values may also be separated by commas, colons, or slashes, and offsets.
-%  Offsets may be prefixed by multiple signs to make offset string
-%  substitutions easier to handle from shell scripts.
-%  For example: "-10-10", "-+10-+10", or "+-10+-10" will generate negtive
-%  offsets, while "+10+10", "++10++10", or "--10--10" will generate positive
-%  offsets.
+%  Chroma subsampling definitions have to be in the form of a:b:c.  Offsets may
+%  be prefixed by multiple signs to make offset string substitutions easier to
+%  handle from shell scripts.  For example: "-10-10", "-+10-+10", or "+-10+-10"
+%  will generate negative offsets, while "+10+10", "++10++10", or "--10--10"
+%  will generate positive offsets.
 %
 %  The format of the ParseGeometry method is:
 %
@@ -964,6 +1016,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       {
         if (*(p+1) == ')')
           return(flags);
+        (void) CopyMagickString(p,p+1,MagickPathExtent);
+        break;
       }
       case ')':
       {
@@ -1230,7 +1284,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %  with respect to the given image page (canvas) dimensions and the images
 %  gravity setting.
 %
-%  This is typically used for specifing a area within a given image for
+%  This is typically used for specifying a area within a given image for
 %  cropping images to a smaller size, chopping out rows and or columns, or
 %  resizing and positioning overlay images.
 %
@@ -1240,7 +1294,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %  The format of the ParseGravityGeometry method is:
 %
 %      MagickStatusType ParseGravityGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info,ExceptionInfo *exception)
+%        RectangleInfo *region_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1262,6 +1316,10 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
     height,
     width;
 
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",geometry);
+  if ((geometry == (char *) NULL) || (*geometry == '\0'))
+    return(NoValue);
   SetGeometry(image,region_info);
   if (image->page.width != 0)
     region_info->width=image->page.width;
@@ -1346,6 +1404,20 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
   if (height == 0)
     region_info->height=image->page.height | image->rows;
   GravityAdjustGeometry(image->columns,image->rows,image->gravity,region_info);
+  if ((flags & LessValue) != 0)
+    if ((region_info->width < image->columns) &&
+        (region_info->height < image->rows))
+      {
+        SetGeometry(image,region_info);
+        return(NoValue);
+      }
+  if ((flags & GreaterValue) != 0)
+    if ((region_info->width > image->columns) &&
+        (region_info->height > image->rows))
+      {
+        SetGeometry(image,region_info);
+        return(NoValue);
+      }
   region_info->width=width;
   region_info->height=height;
   return(flags);
@@ -1404,8 +1476,8 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
     flags;
 
   size_t
-    former_height,
-    former_width;
+    stasis_height,
+    stasis_width;
 
   /*
     Ensure the image geometry is valid.
@@ -1414,15 +1486,16 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
   assert(y != (ssize_t *) NULL);
   assert(width != (size_t *) NULL);
   assert(height != (size_t *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",geometry);
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(NoValue);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",geometry);
   /*
     Parse geometry using GetGeometry.
   */
+  stasis_width=(*width);
+  stasis_height=(*height);
   SetGeometryInfo(&geometry_info);
-  former_width=(*width);
-  former_height=(*height);
   flags=GetGeometry(geometry,x,y,width,height);
   if ((flags & PercentValue) != 0)
     {
@@ -1442,10 +1515,10 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       scale.y=geometry_info.sigma;
       if ((percent_flags & SigmaValue) == 0)
         scale.y=scale.x;
-      *width=(size_t) floor(scale.x*former_width/100.0+0.5);
-      *height=(size_t) floor(scale.y*former_height/100.0+0.5);
-      former_width=(*width);
-      former_height=(*height);
+      *width=(size_t) floor(scale.x*stasis_width/100.0+0.5);
+      *height=(size_t) floor(scale.y*stasis_height/100.0+0.5);
+      stasis_width=(*width);
+      stasis_height=(*height);
     }
   if ((flags & AspectRatioValue) != 0)
     {
@@ -1458,30 +1531,30 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       */
       (void) ParseGeometry(geometry,&geometry_info);
       geometry_ratio=geometry_info.rho;
-      image_ratio=(double) former_width*
-        PerceptibleReciprocal((double) former_height);
+      image_ratio=(double) stasis_width*PerceptibleReciprocal((double)
+        stasis_height);
       if (geometry_ratio >= image_ratio)
         {
-          *width=former_width;
+          *width=stasis_width;
           *height=(size_t) floor((double) (PerceptibleReciprocal(
-            geometry_ratio)*former_height*image_ratio)+0.5);
+            geometry_ratio)*stasis_height*image_ratio)+0.5);
         }
       else
         {
-          *width=(size_t) floor((double) (PerceptibleReciprocal(
-            image_ratio)*former_width*geometry_ratio)+0.5);
-          *height=former_height;
+          *width=(size_t) floor((double) (PerceptibleReciprocal(image_ratio)*
+            stasis_width*geometry_ratio)+0.5);
+          *height=stasis_height;
         }
-      former_width=(*width);
-      former_height=(*height);
+      stasis_width=(*width);
+      stasis_height=(*height);
     }
-  if (((flags & AspectValue) != 0) || ((*width == former_width) &&
-      (*height == former_height)))
+  if (((flags & AspectValue) != 0) || ((*width == stasis_width) &&
+      (*height == stasis_height)))
     {
       if ((flags & RhoValue) == 0)
-        *width=former_width;
+        *width=stasis_width;
       if ((flags & SigmaValue) == 0)
-        *height=former_height;
+        *height=stasis_height;
     }
   else
     {
@@ -1491,52 +1564,52 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       /*
         Respect aspect ratio of the image.
       */
-      if ((former_width == 0) || (former_height == 0))
+      if ((stasis_width == 0) || (stasis_height == 0))
         scale_factor=1.0;
       else
         if (((flags & RhoValue) != 0) && (flags & SigmaValue) != 0)
           {
-            scale_factor=(double) *width/(double) former_width;
+            scale_factor=(double) *width/(double) stasis_width;
             if ((flags & MinimumValue) == 0)
               {
-                if (scale_factor > ((double) *height/(double) former_height))
-                  scale_factor=(double) *height/(double) former_height;
+                if (scale_factor > ((double) *height/(double) stasis_height))
+                  scale_factor=(double) *height/(double) stasis_height;
               }
             else
-              if (scale_factor < ((double) *height/(double) former_height))
-                scale_factor=(double) *height/(double) former_height;
+              if (scale_factor < ((double) *height/(double) stasis_height))
+                scale_factor=(double) *height/(double) stasis_height;
           }
         else
           if ((flags & RhoValue) != 0)
             {
-              scale_factor=(double) *width/(double) former_width;
+              scale_factor=(double) *width/(double) stasis_width;
               if (((flags & MinimumValue) != 0) &&
-                  (scale_factor < ((double) *width/(double) former_height)))
-                scale_factor=(double) *width/(double) former_height;
+                  (scale_factor < ((double) *width/(double) stasis_height)))
+                scale_factor=(double) *width/(double) stasis_height;
             }
           else
             {
-              scale_factor=(double) *height/(double) former_height;
+              scale_factor=(double) *height/(double) stasis_height;
               if (((flags & MinimumValue) != 0) &&
-                  (scale_factor < ((double) *height/(double) former_width)))
-                scale_factor=(double) *height/(double) former_width;
+                  (scale_factor < ((double) *height/(double) stasis_width)))
+                scale_factor=(double) *height/(double) stasis_width;
             }
-      *width=MagickMax((size_t) floor(scale_factor*former_width+0.5),1UL);
-      *height=MagickMax((size_t) floor(scale_factor*former_height+0.5),1UL);
+      *width=MagickMax((size_t) floor(scale_factor*stasis_width+0.5),1UL);
+      *height=MagickMax((size_t) floor(scale_factor*stasis_height+0.5),1UL);
     }
   if ((flags & GreaterValue) != 0)
     {
-      if (former_width < *width)
-        *width=former_width;
-      if (former_height < *height)
-        *height=former_height;
+      if (stasis_width < *width)
+        *width=stasis_width;
+      if (stasis_height < *height)
+        *height=stasis_height;
     }
   if ((flags & LessValue) != 0)
     {
-      if (former_width > *width)
-        *width=former_width;
-      if (former_height > *height)
-        *height=former_height;
+      if (stasis_width > *width)
+        *width=stasis_width;
+      if (stasis_height > *height)
+        *height=stasis_height;
     }
   if ((flags & AreaValue) != 0)
     {
@@ -1552,20 +1625,18 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       */
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho+sqrt(MagickEpsilon);
-      distance=sqrt((double) former_width*former_height);
-      scale.x=(double) former_width*PerceptibleReciprocal(distance*
+      distance=sqrt((double) stasis_width*stasis_height);
+      scale.x=(double) stasis_width*PerceptibleReciprocal(distance*
         PerceptibleReciprocal(sqrt(area)));
-      scale.y=(double) former_height*PerceptibleReciprocal(distance*
+      scale.y=(double) stasis_height*PerceptibleReciprocal(distance*
         PerceptibleReciprocal(sqrt(area)));
       if ((scale.x < (double) *width) || (scale.y < (double) *height))
         {
-          *width=(unsigned long) (former_width*PerceptibleReciprocal(
+          *width=(unsigned long) (stasis_width*PerceptibleReciprocal(
             distance*PerceptibleReciprocal(sqrt(area))));
-          *height=(unsigned long) (former_height*PerceptibleReciprocal(
+          *height=(unsigned long) (stasis_height*PerceptibleReciprocal(
             distance*PerceptibleReciprocal(sqrt(area))));
         }
-      former_width=(*width);
-      former_height=(*height);
     }
   return(flags);
 }
@@ -1590,7 +1661,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
 %  The format of the ParsePageGeometry method is:
 %
 %      MagickStatusType ParsePageGeometry(const Image *image,
-%        const char *geometry,RectangeInfo *region_info,
+%        const char *geometry,RectangleInfo *region_info,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -1660,7 +1731,7 @@ MagickExport MagickStatusType ParsePageGeometry(const Image *image,
 %  The format of the ParseRegionGeometry method is:
 %
 %      MagickStatusType ParseRegionGeometry(const Image *image,
-%        const char *geometry,RectangeInfo *region_info,
+%        const char *geometry,RectangleInfo *region_info,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -1715,7 +1786,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (RectangleInfo *) NULL);
   (void) memset(geometry,0,sizeof(*geometry));
@@ -1748,6 +1819,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
 {
   assert(geometry_info != (GeometryInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   (void) memset(geometry_info,0,sizeof(*geometry_info));
 }
