@@ -47,6 +47,7 @@
 #include "MagickCore/artifact.h"
 #include "MagickCore/attribute.h"
 #include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
 #include "MagickCore/cache.h"
 #include "MagickCore/client.h"
 #include "MagickCore/coder.h"
@@ -143,7 +144,7 @@ static ChannelStatistics *GetLocationStatistics(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   channel_statistics=(ChannelStatistics *) AcquireQuantumMemory(
     MaxPixelChannels+1,sizeof(*channel_statistics));
@@ -503,6 +504,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   const MagickInfo
     *magick_info;
 
+  const Quantum
+    *p;
+
   double
     elapsed_time,
     scale,
@@ -514,23 +518,21 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   MagickBooleanType
     ping;
 
-  const Quantum
-    *p;
-
-  ssize_t
-    i,
-    x;
-
   size_t
     depth,
     distance;
 
   ssize_t
+    i,
+    x,
     y;
 
+  struct stat
+    properties;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (file == (FILE *) NULL)
     file=stdout;
@@ -595,8 +597,13 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         default:
         {
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
-            (void) PrintChannelLocations(file,image,(PixelChannel) i,"Channel",
+          {
+            PixelChannel channel = GetPixelChannelChannel(image,i);
+            if (channel == AlphaPixelChannel)
+              continue;
+            (void) PrintChannelLocations(file,image,channel,"Channel",
               statistic_type,max_locations,channel_statistics);
+          }
           break;
         }
       }
@@ -718,6 +725,16 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         GetPathComponent(image->magick_filename,TailPath,filename);
         (void) FormatLocaleFile(file,"  Base filename: %s\n",filename);
       }
+  properties=(*GetBlobProperties(image));
+  if (properties.st_mode != 0)
+    {
+      static const char *rwx[] =
+        { "---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"};
+      (void) FormatLocaleFile(file,"  Permissions: %s%s%s\n",
+        rwx[(properties.st_mode >> 6) & 0x07],
+        rwx[(properties.st_mode >> 3) & 0x07],
+        rwx[(properties.st_mode >> 0) & 0x07]);
+    }
   magick_info=GetMagickInfo(image->magick,exception);
   if ((magick_info == (const MagickInfo *) NULL) ||
       (GetMagickDescription(magick_info) == (const char *) NULL))
@@ -768,6 +785,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (void) FormatLocaleFile(file,"  Depth: %.20g/%.20g-bit\n",(double)
           image->depth,(double) depth);
     }
+  (void) FormatLocaleFile(file,"  Channels: %g.%g\n",(double)
+    image->number_channels,(double) image->number_meta_channels);
   if (channel_statistics != (ChannelStatistics *) NULL)
     {
       /*
@@ -809,8 +828,16 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         default:
         {
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+          {
+            PixelChannel channel = GetPixelChannelChannel(image,i);
+            if ((channel == AlphaPixelChannel) ||
+                (channel == ReadMaskPixelChannel) ||
+                (channel == WriteMaskPixelChannel) ||
+                (channel == CompositeMaskPixelChannel))
+              continue;
             (void) FormatLocaleFile(file,"    Channel %.20g: %.20g-bit\n",
-              (double) i,(double) channel_statistics[i].depth);
+              (double) i,(double) channel_statistics[channel].depth);
+          }
           break;
         }
       }
@@ -824,12 +851,17 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (void) FormatLocaleFile(file,"    Write mask: %.20g-bit\n",(double)
           channel_statistics[WriteMaskPixelChannel].depth);
       if ((image->channels & CompositeMaskChannel) != 0)
-        (void) FormatLocaleFile(file,"    Composite mask: %.20g-bit\n",(double)
-          channel_statistics[CompositeMaskPixelChannel].depth);
+        (void) FormatLocaleFile(file,"    Composite mask: %.20g-bit\n",
+          (double) channel_statistics[CompositeMaskPixelChannel].depth);
       if (image->number_meta_channels != 0)
         for (i=0; i < (ssize_t) image->number_meta_channels; i++)
+        {
+          PixelChannel
+            channel = (PixelChannel) (MetaPixelChannels+i);
+
           (void) FormatLocaleFile(file,"    Meta channel[%.20g]: %.20g-bit\n",
-            (double) i,(double) channel_statistics[MetaPixelChannels+i].depth);
+            (double) i,(double) channel_statistics[channel].depth);
+        }
       scale=1.0;
       if (image->depth <= MAGICKCORE_QUANTUM_DEPTH)
         scale=(double) (QuantumRange/((size_t) QuantumRange >> ((size_t)
@@ -874,25 +906,31 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
           {
             char
-              channel[MagickPathExtent];
+              label[MagickPathExtent];
 
-            (void) FormatLocaleString(channel,MagickPathExtent,"Channel %.20g",
+            PixelChannel channel = GetPixelChannelChannel(image,i);
+            if ((channel == AlphaPixelChannel) ||
+                (channel == ReadMaskPixelChannel) ||
+                (channel == WriteMaskPixelChannel) ||
+                (channel == CompositeMaskPixelChannel))
+              continue;
+            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
               (double) i);
-            (void) PrintChannelStatistics(file,(PixelChannel) i,channel,1.0/
-              scale,channel_statistics);
+            (void) PrintChannelStatistics(file,channel,label,1.0/scale,
+              channel_statistics);
           }
           break;
         }
       }
       if (image->alpha_trait != UndefinedPixelTrait)
-        (void) PrintChannelStatistics(file,AlphaPixelChannel,"Alpha",1.0/scale,
-          channel_statistics);
+        (void) PrintChannelStatistics(file,AlphaPixelChannel,
+          "Alpha",1.0/scale,channel_statistics);
       if ((image->channels & ReadMaskChannel) != 0)
-        (void) PrintChannelStatistics(file,ReadMaskPixelChannel,"Read mask",
-          1.0/scale,channel_statistics);
+        (void) PrintChannelStatistics(file,ReadMaskPixelChannel,
+          "Read mask",1.0/scale,channel_statistics);
       if ((image->channels & WriteMaskChannel) != 0)
-        (void) PrintChannelStatistics(file,WriteMaskPixelChannel,"Write mask",
-          1.0/scale,channel_statistics);
+        (void) PrintChannelStatistics(file,WriteMaskPixelChannel,
+          "Write mask",1.0/scale,channel_statistics);
       if ((image->channels & CompositeMaskChannel) != 0)
         (void) PrintChannelStatistics(file,WriteMaskPixelChannel,
           "Composite mask",1.0/scale,channel_statistics);
@@ -902,10 +940,13 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           char
             label[MagickPathExtent];
 
+          PixelChannel
+            channel = (PixelChannel) (MetaPixelChannels+i);
+
           (void) FormatLocaleString(label,MagickPathExtent,
             "Meta channel[%.20g]",(double) i);
-          (void) PrintChannelStatistics(file,(const PixelChannel)
-            (MetaPixelChannels+i),label,1.0/scale,channel_statistics);
+          (void) PrintChannelStatistics(file,channel,label,1.0/scale,
+            channel_statistics);
         }
       if ((colorspace != LinearGRAYColorspace) &&
           (colorspace != GRAYColorspace))
@@ -957,11 +998,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
           {
             char
-              channel[MagickPathExtent];
+              label[MagickPathExtent];
 
-            (void) FormatLocaleString(channel,MagickPathExtent,"Channel %.20g",
+            PixelChannel channel = GetPixelChannelChannel(image,i);
+            if (channel == AlphaPixelChannel)
+              continue;
+            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
               (double) i);
-            (void) PrintChannelMoments(file,(PixelChannel) i,"channel",scale,
+            (void) PrintChannelMoments(file,channel,label,scale,
               channel_moments);
           }
           break;
@@ -1025,12 +1069,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
           {
             char
-              channel[MagickPathExtent];
+              label[MagickPathExtent];
 
-            (void) FormatLocaleString(channel,MagickPathExtent,"Channel %.20g",
+            PixelChannel channel = GetPixelChannelChannel(image,i);
+            if (channel == AlphaPixelChannel)
+              continue;
+            (void) FormatLocaleString(label,MagickPathExtent,"Channel %.20g",
               (double) i);
-            (void) PrintChannelFeatures(file,(PixelChannel) i,channel,
-              channel_features);
+            (void) PrintChannelFeatures(file,channel,label,channel_features);
           }
           break;
         }
@@ -1562,6 +1608,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   if (strlen(buffer) > 1)
     buffer[strlen(buffer)-1]='\0';
   (void) FormatLocaleFile(file,"  Number pixels: %s\n",buffer);
+  (void) FormatLocaleString(buffer,MagickPathExtent,"%s",
+    CommandOptionToMnemonic(MagickCacheOptions,(ssize_t)
+    GetImagePixelCacheType(image)));
+  (void) FormatLocaleFile(file,"  Pixel cache type: %s\n",buffer);
   if (elapsed_time > MagickEpsilon)
     {
       (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
