@@ -69,6 +69,7 @@
 #include "MagickCore/module.h"
 #include "MagickCore/transform.h"
 #include "MagickCore/utility.h"
+#include "MagickCore/utility-private.h"
 #include "MagickCore/xml-tree.h"
 #include "MagickCore/xml-tree-private.h"
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
@@ -183,7 +184,7 @@ static void SetDNGProperties(Image *image,const libraw_data_t *raw_info,
     raw_info->other.aperture);
   (void) FormatImageProperty(image,"dng:max.aperture.value","%0.1f",
     raw_info->lens.EXIF_MaxAp);
-  (void) FormatImageProperty(image,"dng:focal.length","%0.1f m",
+  (void) FormatImageProperty(image,"dng:focal.length","%0.1f mm",
     raw_info->other.focal_len);
   (void) FormatImageProperty(image,"dng:wb.rb.levels","%f %f %f %f",
     raw_info->color.cam_mul[0],raw_info->color.cam_mul[2],
@@ -331,6 +332,7 @@ static void SetLibRawParams(const ImageInfo *image_info,Image *image,
   const char
     *option;
 
+  raw_info->params.user_flip=0;
   raw_info->params.output_bps=16;
   option=GetImageOption(image_info,"dng:use-camera-wb");
   if (option == (const char *) NULL)
@@ -356,6 +358,7 @@ static void SetLibRawParams(const ImageInfo *image_info,Image *image,
       if (raw_info->params.output_color == 5)
         image->colorspace=XYZColorspace;
     }
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,21)
   option=GetImageOption(image_info,"dng:interpolation-quality");
   if (option != (const char *) NULL)
     {
@@ -368,6 +371,7 @@ static void SetLibRawParams(const ImageInfo *image_info,Image *image,
       else
         raw_info->params.user_qual=value;
     }
+#endif
 }
 
 static void LibRawDataError(void *data,const char *magick_unused(file),
@@ -419,6 +423,19 @@ static void ReadLibRawThumbnail(const ImageInfo *image_info,Image *image,
     libraw_dcraw_clear_mem(thumbnail);
 }
 
+static OrientationType LibRawFlipToOrientation(int flip)
+{
+  switch(flip)
+  {
+    case 5:
+      return(LeftBottomOrientation);
+    case 8:
+      return(LeftTopOrientation);
+    default:
+      return((OrientationType) flip);
+  }
+}
+
 #endif
 
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -466,12 +483,18 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     StringInfo
       *profile;
 
+    unsigned int
+      flags;
+
     unsigned short
       *p;
 
-    errcode=0;
-    raw_info=libraw_init(LIBRAW_OPIONS_NO_MEMERR_CALLBACK |
-      LIBRAW_OPIONS_NO_DATAERR_CALLBACK);
+    errcode=LIBRAW_UNSPECIFIED_ERROR;
+    flags=LIBRAW_OPIONS_NO_DATAERR_CALLBACK;
+#if LIBRAW_SHLIB_CURRENT < 23
+    flags|=LIBRAW_OPIONS_NO_MEMERR_CALLBACK;
+#endif
+    raw_info=libraw_init(flags);
     if (raw_info == (libraw_data_t *) NULL)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
@@ -483,11 +506,14 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1310)
     {
       wchar_t
-        fileName[MagickPathExtent];
+        *path;
 
-      MultiByteToWideChar(CP_UTF8,0,image->filename,-1,fileName,
-        MagickPathExtent);
-      errcode=libraw_open_wfile(raw_info,fileName);
+      path=create_wchar_path(image->filename);
+      if (path != (wchar_t *) NULL)
+        {
+          errcode=libraw_open_wfile(raw_info,path);
+          path=(wchar_t *) RelinquishMagickMemory(path);
+        }
     }
 #else
     errcode=libraw_open_file(raw_info,image->filename);
@@ -505,7 +531,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.height=raw_info->sizes.raw_height;
     image->page.x=raw_info->sizes.left_margin;
     image->page.y=raw_info->sizes.top_margin;
-    image->orientation=(OrientationType) raw_info->sizes.flip;
+    image->orientation=LibRawFlipToOrientation(raw_info->sizes.flip);
     ReadLibRawThumbnail(image_info,image,raw_info,exception);
     if (image_info->ping != MagickFalse)
       {
