@@ -445,8 +445,7 @@ static boolean ReadProfileData(j_decompress_ptr jpeg_info,const int index,
     }
   if (client_info->profiles[index] == (StringInfo *) NULL)
     {
-      client_info->profiles[index]=BlobToStringInfo((const void *) NULL,
-        length);
+      client_info->profiles[index]=BlobToStringInfo((const void *) NULL,length);
       if (client_info->profiles[index] == (StringInfo *) NULL)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
@@ -461,8 +460,7 @@ static boolean ReadProfileData(j_decompress_ptr jpeg_info,const int index,
         current_length;
 
       current_length=GetStringInfoLength(client_info->profiles[index]);
-      SetStringInfoLength(client_info->profiles[index],current_length+
-        length);
+      SetStringInfoLength(client_info->profiles[index],current_length+length);
       p=GetStringInfoDatum(client_info->profiles[index])+current_length;
     }
   for (i=0; i < (ssize_t) length; i++)
@@ -1218,7 +1216,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
           "Scale factor: %.20g",(double) scale_factor);
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
-#if defined(D_LOSSLESS_SUPPORTED)
+#if !defined(LIBJPEG_TURBO_VERSION_NUMBER) && defined(D_LOSSLESS_SUPPORTED)
   image->interlace=jpeg_info->process == JPROC_PROGRESSIVE ?
     JPEGInterlace : NoInterlace;
   image->compression=jpeg_info->process == JPROC_LOSSLESS ?
@@ -2228,7 +2226,8 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     *value;
 
   Image
-    *volatile volatile_image;
+    *jps_image = (Image *) NULL,
+    *volatile volatile_image = (Image *) NULL;
 
   int
     colorspace,
@@ -2277,10 +2276,18 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((LocaleCompare(image_info->magick,"JPS") == 0) &&
       (image->next != (Image *) NULL))
-    image=AppendImages(image,MagickFalse,exception);
+    {
+      jps_image=AppendImages(image,MagickFalse,exception);
+      if (jps_image != (Image *) NULL)
+        image=jps_image;
+    }
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
-    return(status);
+    {
+      if (jps_image != (Image *) NULL)
+        jps_image=DestroyImage(jps_image);
+      return(status);
+    }
   /*
     Initialize JPEG parameters.
   */
@@ -2302,7 +2309,9 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     {
       jpeg_destroy_compress(jpeg_info);
       client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
-      (void) CloseBlob(volatile_image);
+      (void) CloseBlob(image);
+      if (jps_image != (Image *) NULL)
+        jps_image=DestroyImage(jps_image);
       return(MagickFalse);
     }
   jpeg_info->client_data=(void *) client_info;
@@ -2470,11 +2479,7 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     }
   else
     {
-#if !defined(C_LOSSLESS_SUPPORTED)
-      quality=100;
-      if (image->debug != MagickFalse)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Quality: 100");
-#else
+#if !defined(LIBJPEG_TURBO_VERSION_NUMBER) && defined(C_LOSSLESS_SUPPORTED)
       if (image->quality < 100)
         (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
           "LosslessToLossyJPEGConversion","`%s'",image->filename);
@@ -2497,6 +2502,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
                 "Point Transform: %d",point_transform);
             }
         }
+#else
+      quality=100;
+      if (image->debug != MagickFalse)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Quality: 100");
 #endif
     }
   option=GetImageOption(image_info,"jpeg:extent");
@@ -2796,6 +2805,8 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
       if (memory_info != (MemoryInfo *) NULL)
         memory_info=RelinquishVirtualMemory(memory_info);
       (void) CloseBlob(image);
+      if (jps_image != (Image *) NULL)
+        jps_image=DestroyImage(jps_image);
       return(MagickFalse);
     }
   scanline[0]=(JSAMPROW) jpeg_pixels;
@@ -2992,6 +3003,8 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
   client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
   memory_info=RelinquishVirtualMemory(memory_info);
   (void) CloseBlob(image);
+  if (jps_image != (Image *) NULL)
+    jps_image=DestroyImage(jps_image);
   return(MagickTrue);
 }
 
